<?php
    /*
    Kolab Z-Push Backend

    Copyright (C) 2009-2010 Free Software Foundation Europe e.V.

    The main author of the Kolab Z-Push Backend is Alain Abbas, with
    contributions by .......

    This program is Free Software; you can redistribute it and/or
    modify it under the terms of version two of the GNU General Public
    License as published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.

    The licensor of the Kolab Z-Push Backend is the 
    Free Software Foundation Europe (FSFE), Fiduciary Program, 
    Linienstr. 141, 10115 Berlin, Germany, email:ftf@fsfeurope.org.
    */                                                                    
    define("KOLABACTIVESYNCDATA_VERSION","20100707")  ;
    abstract class KolabZpushParam
    {
        protected $data;
        protected $olddata;
        public function serialize()
        {
            if (function_exists("json_encode"))
            {
                return base64_encode(json_encode($this->data));
            }
            else
            {
                $json=new FastJSON();
                return base64_encode($json->encode($this->data));
            }
        }
        protected function unserialize($base64encodedData)
        {
            $encodedData=base64_decode($base64encodedData);
            $this->olddata=$encodedData;
            if (function_exists("json_decode"))
            {
                $this->data= json_decode($encodedData, true);
            }
            else
            {
                $json=new FastJSON();
                $this->data=$json->decode($encodedData);
                if ($json->error == true)
                {
                    $this->data=false;
                }
            }
        }
        public function isModified()
        {
            $d=$this->serialize();
            if ($d != $this->olddata)
            {
                return true;
            }
            return false;

        }
    }
    class GlobalParam extends KolabZpushParam
    {
     /**
     * public  method
     *
     *    GlobalParam::unserialize(params:String ):Boolean
     *
     * @param    String    String JSON formatted
     * @return   Boolean   false or true.
     *
     * @note   load the object with serialized json string
     */
        public function unserialize($encodedData)  
        {
            parent::unserialize($encodedData);
            if (! isset($this->data["DEVICE"]))
            {
                return false;
            }    
        }
        
     /**
     * public  method
     *
     *    GlobalParam::deviceList():Array
     *
     * @return   Array   Array of serial device present
     *
     * @note   return the list of the serial 
     */
        public function deviceList()
        {
            $devicetab=array();
            foreach( $this->data["DEVICE"] as $key => $dev)
            {
                $devicetab[]=$key;
            }
            return $devicetab;    
        }
     /**
     * public  method
     *
     *    GlobalParam::getDeviceType(params:String ):String
     *
     * @param    String   Serial number of a Device
     * @return   String   string the type of the device 
     *
     * @note   
     */
        public function getDeviceType($serial)
        {
            if (isset($this->data["DEVICE"][$serial]["TYPE"]))
            {
                return $this->data["DEVICE"][$serial]["TYPE"];
            }
            else
            {
                return "";
            }
        }
         /**
     * public  method
     *
     *    GlobalParam::getDeviceMode(params:String ):Integer
     *
     * @param    String   Serial number of a Device
     * @return   Integer  integer  -1 | 0 | 1  (not set | flatmode | foldermode)
     *
     * @note   
     */
        public function getDeviceMode($serial)
        {
            if (isset($this->data["DEVICE"][$serial]["MODE"]))
            {
                return $this->data["DEVICE"][$serial]["MODE"];
            }
            else
            {
                return -1;
            }  
        }
         /**
     * public  method
     *
     *    GlobalParam::setDevice(params:String,String[,Integer] )
     *
     * @param    *   String Serial number of a Device
     * @param    *   String device type
     * @param    *   Integer 
     *
     * @note   
     */
        public function setDevice($serial,$deviceType,$mode = -1)
        {
            $this->data["DEVICE"][$serial]["TYPE"]=$deviceType;
            $this->data["DEVICE"][$serial]["MODE"]=$mode;  
        }
            /**
     * public  method
     *
     *    GlobalParam::setMode(params:String,,Integer] )
     *
     * @param    String Serial number of a Device
     * @param    Integer Mode
     *
     * @note   
     */
        public function setMode($serial,$mode)
        {
            if (isset($this->data["DEVICE"][$serial]))
            {
                $this->data["DEVICE"][$serial]["MODE"]=$mode;     
            }
        }

    }
    class FolderParam  extends KolabZpushParam
    {
        function unserialize($encodedData)  
        {
            parent::unserialize($encodedData);
            if (! isset($this->data["FOLDER"]))
            {
                return false;
            }    
        }
         public function deviceList()
        {
            $devicetab=array();
            if ( isset($this->data["FOLDER"]))
            {
                foreach( $this->data["FOLDER"] as $key => $dev)
                {
                    $devicetab[]=$key;
                }
                return $devicetab;
            }
            else
            {
                return false;
            }    
        }
            /**
     * public  method
     *
     *    FolderParam::SetFolder(params:String )
     *
     * @param    *   String Folder name
     *
     * @note    this param is used to determine the default sync if sync is not set
     */
        public function setFolder($folder)
        {
            $this->folder=$folder;
        }
                /**
     * public  method
     *
     *    FolderParam::isForSync(params:String ):Boolean
     *
     * @param    *   String Device Serial
     *
     * @return   Boolean 0|1
     * @note    
     */
        public function isForSync($serial)
        {
            if (isset($this->data["FOLDER"][$serial]["S"] ))
            {
                if ($this->data["FOLDER"][$serial]["S"] > 0 )
                {
                    return 1;
                }
                else
                {
                    return 0;
                }
            }
            else
            {
                if (substr($this->folder,0,5) == "INBOX")
                {
                    return 1;
                }
                else
                {
                    return 0;
                }
            }
        }
        /** public  method
     *
     *    FolderParam::SetForSync(params:String )
     *
     * @param    *   String Device Serial
     *
     * @note    set the folder for Sync 
     */
        public function setForSync($serial)
        {
            if (substr($this->folder,0,5) == "INBOX")
            {
                $this->data["FOLDER"][$serial]["S"]=2;
            }
            else
            {
                $this->data["FOLDER"][$serial]["S"]=1; 
            }
        }
           /** public  method
     *
     *    FolderParam::unsetSetForSync(params:String )
     *
     * @param    *   String Device Serial
     *
     * @note    unset the folder for Sync 
     */
        public function unsetForSync($serial)
        {
            $this->data["FOLDER"][$serial]["S"]=0;
        }
           /** public  method
     *
     *    FolderParam::showAlarm(params:String )
     *
     * @param    *   String Device Serial
     *
     * @note    Alarm must be show for this folder
     */
        public function showAlarm($serial)
        {
            if (isset($this->data["FOLDER"][$serial]["S"] ))
            {
                if ($this->data["FOLDER"][$serial]["S"] == 2)
                {
                    return 1;
                }
                else
                {
                    return 0;
                }
            }
            else
            {
                if (substr($this->folder,0,5) == "INBOX")
                {
                    return 1;
                }
                else
                {
                    return 0;
                }
            }
        }
                  /** public  method
     *
     *    FolderParam::setAlarm(params:String )
     *
     * @param    *   String Device Serial
     *
     * @note    set alarm for this folder
     */
        public function setAlarm($serial)
        {
            $this->data["FOLDER"][$serial]["S"]=2;            
        }
        
        public function unsetAlarm($serial)
        {
            $this->data["FOLDER"][$serial]["S"]=1;      
        }
    }
    class FastJSON {

    // public methods
    var $error=false; 
    /**
     * public static method
     *
     *    FastJSON::convert(params:* [, result:Instance]):*
     *
     * @param    *    String or Object
     * @param    Instance    optional new generic class instance if first
     *                parameter is an object.
     * @return    *    time() value or new Instance with object parameters.
     *
     * @note    please read Special FastJSON::convert method Informations
     */
    static public function convert($params, $result = null){
        switch(gettype($params)){
            case    'array':
                    $tmp = array();
                    foreach($params as $key => $value) {
                        if(($value = FastJSON::encode($value)) !== '')
                            array_push($tmp, FastJSON::encode(strval($key)).':'.$value);
                    };
                    $result = '{'.implode(',', $tmp).'}';
                    break;
            case    'boolean':
                    $result = $params ? 'true' : 'false';
                    break;
            case    'double':
            case    'float':
            case    'integer':
                    $result = $result !== null ? strftime('%Y-%m-%dT%H:%M:%S', $params) : strval($params);
                    break;
            case    'NULL':
                    $result = 'null';
                    break;
            case    'string':
                    $i = create_function('&$e, $p, $l', 'return intval(substr($e, $p, $l));');
                    if(preg_match('/^[0-9]{4}\-[0-9]{2}\-[0-9]{2}T[0-9]{2}:[0-9]{2}:[0-9]{2}$/', $params))
                        $result = mktime($i($params, 11, 2), $i($params, 14, 2), $i($params, 17, 2), $i($params, 5, 2), $i($params, 9, 2), $i($params, 0, 4));
                    break;
            case    'object':
                    $tmp = array();
                    if(is_object($result)) {
                        foreach($params as $key => $value)
                            $result->$key = $value;
                    } else {
                        $result = get_object_vars($params);
                        foreach($result as $key => $value) {
                            if(($value = FastJSON::encode($value)) !== '')
                                array_push($tmp, FastJSON::encode($key).':'.$value);
                        };
                        $result = '{'.implode(',', $tmp).'}';
                    }
                    break;
        }
        return $result;
    }

    /**
     * public method
     *
     *    FastJSON::decode(params:String[, useStdClass:Boolean]):*
     *
     * @param    String    valid JSON encoded string
     * @param    Bolean    uses stdClass instead of associative array if params contains objects (default false)
     * @return    *    converted variable or null
     *                is params is not a JSON compatible string.
     * @note    This method works in an optimist way. If JSON string is not valid
     *         the code execution will die using exit.
     *        This is probably not so good but JSON is often used combined with
     *        XMLHttpRequest then I suppose that's better more protection than
     *        just some WARNING.
     *        With every kind of valid JSON string the old error_reporting level
     *        and the old error_handler will be restored.
     *
     * @example
     *        FastJSON::decode('{"param":"value"}'); // associative array
     *        FastJSON::decode('{"param":"value"}', true); // stdClass
     *        FastJSON::decode('["one",two,true,false,null,{},[1,2]]'); // array
     */
    static public function decode($encode, $stdClass = false){
        $pos = 0;
        $slen = is_string($encode) ? strlen($encode) : null;
        if($slen !== null) {
            $error = error_reporting(0);
            set_error_handler(array('FastJSON', '__exit'));
            $result = FastJSON::__decode($encode, $pos, $slen, $stdClass);
            error_reporting($error);
            restore_error_handler();
        }
        else
            $result = null;
        return $result;
    }

    /**
     * public method
     *
     *    FastJSON::encode(params:*):String
     *
     * @param    *        Array, Boolean, Float, Int, Object, String or NULL variable.
     * @return    String        JSON genric object rappresentation
     *                or empty string if param is not compatible.
     *
     * @example
     *        FastJSON::encode(array(1,"two")); // '[1,"two"]'
     *
     *        $obj = new MyClass();
     *        obj->param = "value";
     *        obj->param2 = "value2";
     *        FastJSON::encode(obj); // '{"param":"value","param2":"value2"}'
     */
    static public function encode($decode){
        $result = '';
        switch(gettype($decode)){
            case    'array':
                    if(!count($decode) || array_keys($decode) === range(0, count($decode) - 1)) {
                        $keys = array();
                        foreach($decode as $value) {
                            if(($value = FastJSON::encode($value)) !== '')
                                array_push($keys, $value);
                        }
                        $result = '['.implode(',', $keys).']';
                    }
                    else
                        $result = FastJSON::convert($decode);
                    break;
            case    'string':
                    $replacement = FastJSON::__getStaticReplacement();
                    $result = '"'.str_replace($replacement['find'], $replacement['replace'], $decode).'"';
                    break;
            default:
                    if(!is_callable($decode))
                        $result = FastJSON::convert($decode);
                    break;
        }
        return $result;
    }

    // private methods, uncommented, sorry
    static private function __getStaticReplacement(){
        static $replacement = array('find'=>array(), 'replace'=>array());
        if($replacement['find'] == array()) {
            foreach(array_merge(range(0, 7), array(11), range(14, 31)) as $v) {
                $replacement['find'][] = chr($v);
                $replacement['replace'][] = "\\u00".sprintf("%02x", $v);
            }
            $replacement['find'] = array_merge(array(chr(0x5c), chr(0x2F), chr(0x22), chr(0x0d), chr(0x0c), chr(0x0a), chr(0x09), chr(0x08)), $replacement['find']);
            $replacement['replace'] = array_merge(array('\\\\', '\\/', '\\"', '\r', '\f', '\n', '\t', '\b'), $replacement['replace']);
        }    
        return $replacement;
    }
    static private function __decode(&$encode, &$pos, &$slen, &$stdClass){
        switch($encode{$pos}) {
            case 't':
                $result = true;
                $pos += 4;
                break;
            case 'f':
                $result = false;
                $pos += 5;
                break;
            case 'n':
                $result = null;
                $pos += 4;
                break;
            case '[':
                $result = array();
                ++$pos;
                while($encode{$pos} !== ']') {
                    array_push($result, FastJSON::__decode($encode, $pos, $slen, $stdClass));
                    if($encode{$pos} === ',')
                        ++$pos;
                }
                ++$pos;
                break;
            case '{':
                $result = $stdClass ? new stdClass : array();
                ++$pos;
                while($encode{$pos} !== '}') {
                    $tmp = FastJSON::__decode($encode, $pos, $slen, $stdClass);;
                    ++$pos;
                    if($stdClass)
                        $result->$tmp = FastJSON::__decode($encode, $pos, $slen, $stdClass);
                    else
                        $result[$tmp] = FastJSON::__decode($encode, $pos, $slen, $stdClass);
                    if($encode{$pos} === ',')
                        ++$pos;
                }
                ++$pos;
                break;
            case '"':
                switch($encode{++$pos}) {
                    case '"':
                        $result = "";
                        break;
                    default:
                        $result = FastJSON::__decodeString($encode, $pos);
                        break;
                }
                ++$pos;
                break;
            default:
                $tmp = '';
                preg_replace('/^(\-)?([0-9]+)(\.[0-9]+)?([eE]\+[0-9]+)?/e', '$tmp = "\\1\\2\\3\\4"', substr($encode, $pos));
                if($tmp !== '') {
                    $pos += strlen($tmp);
                    $nint = intval($tmp);
                    $nfloat = floatval($tmp);
                    $result = $nfloat == $nint ? $nint : $nfloat;
                }
                break;
        }
        return $result;
    }
    static private function __decodeString(&$encode, &$pos) {
        $replacement = FastJSON::__getStaticReplacement();
        $endString = FastJSON::__endString($encode, $pos, $pos);
        $result = str_replace($replacement['replace'], $replacement['find'], substr($encode, $pos, $endString));
        $pos += $endString;
        return $result;
    }
    static private function __endString(&$encode, $position, &$pos) {
        do {
            $position = strpos($encode, '"', $position + 1);
        }while($position !== false && FastJSON::__slashedChar($encode, $position - 1));
        if($position === false)
            trigger_error('', E_USER_WARNING);
        return $position - $pos;
    }
    static private function __exit($str, $a, $b) {
        //exit($a.'FATAL: FastJSON decode method failure [malicious or incorrect JSON string]');
        $error = true;
    }
    static private function __slashedChar(&$encode, $position) {
        $pos = 0;
        while($encode{$position--} === '\\')
            $pos++;
        return $pos % 2;
    }
}
?>
